@@ -35,27 +35,42 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._get(_snapshot.Snapshot, snapshot)
 
+    def find_snapshot(self, name_or_id, ignore_missing=True, **attrs):
+        """Find a single snapshot
+
+        :param snapshot: The name or ID a snapshot
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the snapshot does not exist.
+
+        :returns: One :class:`~openstack.volume.v3.snapshot.Snapshot`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._find(_snapshot.Snapshot, name_or_id,
+                          ignore_missing=ignore_missing)
+
     def snapshots(self, details=True, **query):
         """Retrieve a generator of snapshots
 
         :param bool details: When set to ``False``
-                    :class:`~openstack.block_storage.v3.snapshot.Snapshot`
-                    objects will be returned. The default, ``True``, will cause
-                    :class:`~openstack.block_storage.v3.snapshot.SnapshotDetail`
-                    objects to be returned.
+            :class:`~openstack.block_storage.v3.snapshot.Snapshot`
+            objects will be returned. The default, ``True``, will cause
+            more attributes to be returned.
         :param kwargs query: Optional query parameters to be sent to limit
             the snapshots being returned.  Available parameters include:
 
             * name: Name of the snapshot as a string.
             * all_projects: Whether return the snapshots in all projects.
+            * project_id: Filter the snapshots by project.
             * volume_id: volume id of a snapshot.
             * status: Value of the status of the snapshot so that you can
                       filter on "available" for example.
 
         :returns: A generator of snapshot objects.
         """
-        snapshot = _snapshot.SnapshotDetail if details else _snapshot.Snapshot
-        return self._list(snapshot, **query)
+        base_path = '/snapshots/detail' if details else None
+        return self._list(_snapshot.Snapshot, base_path=base_path, **query)
 
     def create_snapshot(self, **attrs):
         """Create a new snapshot from attributes
@@ -98,6 +113,21 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._get(_type.Type, type)
 
+    def find_type(self, name_or_id, ignore_missing=True, **attrs):
+        """Find a single volume type
+
+        :param snapshot: The name or ID a volume type
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the type does not exist.
+
+        :returns: One :class:`~openstack.volume.v3.type.Type`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._find(_type.Type, name_or_id,
+                          ignore_missing=ignore_missing)
+
     def types(self, **query):
         """Retrieve a generator of volume types
 
@@ -132,6 +162,19 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         self._delete(_type.Type, type, ignore_missing=ignore_missing)
 
+    def update_type(self, type, **attrs):
+        """Update a type
+
+        :param type: The value can be either the ID of a type or a
+                     :class:`~openstack.volume.v3.type.Type` instance.
+        :param dict attrs: The attributes to update on the type
+                           represented by ``value``.
+
+        :returns: The updated type
+        :rtype: :class:`~openstack.volume.v3.type.Type`
+        """
+        return self._update(_type.Type, type, **attrs)
+
     def get_volume(self, volume):
         """Get a single volume
 
@@ -144,6 +187,21 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._get(_volume.Volume, volume)
 
+    def find_volume(self, name_or_id, ignore_missing=True, **attrs):
+        """Find a single volume
+
+        :param snapshot: The name or ID a volume
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the volume does not exist.
+
+        :returns: One :class:`~openstack.volume.v3.volume.Volume`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        return self._find(_volume.Volume, name_or_id,
+                          ignore_missing=ignore_missing)
+
     def volumes(self, details=True, **query):
         """Retrieve a generator of volumes
 
@@ -255,6 +313,25 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             )
         return self._get(_backup.Backup, backup)
 
+    def find_backup(self, name_or_id, ignore_missing=True, **attrs):
+        """Find a single backup
+
+        :param snapshot: The name or ID a backup
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the backup does not exist.
+
+        :returns: One :class:`~openstack.volume.v3.backup.Backup`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+                 when no resource can be found.
+        """
+        if not self._connection.has_service('object-store'):
+            raise exceptions.SDKException(
+                'Object-store service is required for block-store backups'
+            )
+        return self._find(_backup.Backup, name_or_id,
+                          ignore_missing=ignore_missing)
+
     def create_backup(self, **attrs):
         """Create a new Backup from attributes with native API
 
@@ -357,23 +434,45 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             }
         }
 
-    def _service_cleanup(self, dry_run=True, status_queue=None):
+    def _service_cleanup(self, dry_run=True, client_status_queue=None,
+                         identified_resources=None,
+                         filters=None, resource_evaluation_fn=None):
         if self._connection.has_service('object-store'):
             # Volume backups require object-store to be available, even for
             # listing
+            backups = []
             for obj in self.backups(details=False):
-                if status_queue:
-                    status_queue.put(obj)
-                if not dry_run:
-                    self.delete_backup(obj)
+                need_delete = self._service_cleanup_del_res(
+                    self.delete_backup,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn)
+                if not dry_run and need_delete:
+                    backups.append(obj)
+
+            # Before deleting snapshots need to wait for backups to be deleted
+            for obj in backups:
+                try:
+                    self.wait_for_delete(obj)
+                except exceptions.SDKException:
+                    # Well, did our best, still try further
+                    pass
 
         snapshots = []
         for obj in self.snapshots(details=False):
-            if status_queue:
+            need_delete = self._service_cleanup_del_res(
+                self.delete_snapshot,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
+            if not dry_run and need_delete:
                 snapshots.append(obj)
-                status_queue.put(obj)
-            if not dry_run:
-                self.delete_snapshot(obj)
 
         # Before deleting volumes need to wait for snapshots to be deleted
         for obj in snapshots:
@@ -383,8 +482,12 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
                 # Well, did our best, still try further
                 pass
 
-        for obj in self.volumes(details=False):
-            if status_queue:
-                status_queue.put(obj)
-            if not dry_run:
-                self.delete_volume(obj)
+        for obj in self.volumes(details=True):
+            self._service_cleanup_del_res(
+                self.delete_volume,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn)
