@@ -108,6 +108,19 @@ class Proxy(proxy.Proxy):
         """
         self._delete(_flavor.Flavor, flavor, ignore_missing=ignore_missing)
 
+    def update_flavor(self, flavor, **attrs):
+        """Update a flavor
+
+        :param server: Either the ID of a flavot or a
+            :class:`~openstack.compute.v2.flavor.Flavor` instance.
+        :attrs kwargs: The attributes to update on the flavor represented
+            by ``flavor``.
+
+        :returns: The updated flavor
+        :rtype: :class:`~openstack.compute.v2.flavor.Flavor`
+        """
+        return self._update(_flavor.Flavor, flavor, **attrs)
+
     def get_flavor(self, flavor, get_extra_specs=False):
         """Get a single flavor
 
@@ -261,6 +274,20 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_aggregate.Aggregate, aggregate)
 
+    def find_aggregate(self, name_or_id, ignore_missing=True):
+        """Find a single aggregate
+
+        :param name_or_id: The name or ID of an aggregate.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the resource does not exist.  When set to ``True``, None will be
+            returned when attempting to find a nonexistent resource.
+        :returns: One :class:`~openstack.compute.v2.aggregate.Aggregate`
+            or None
+        """
+        return self._find(_aggregate.Aggregate, name_or_id,
+                          ignore_missing=ignore_missing)
+
     def create_aggregate(self, **attrs):
         """Create a new host aggregate from attributes
 
@@ -345,6 +372,26 @@ class Proxy(proxy.Proxy):
         aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
         return aggregate.set_metadata(self, metadata)
 
+    def aggregate_precache_images(self, aggregate, images):
+        """Requests image precaching on an aggregate
+
+        :param aggregate: Either the ID of a aggregate or a
+            :class:`~openstack.compute.v2.aggregate.Aggregate` instance.
+        :param images: Single image id or list of image ids.
+
+        :returns: ``None``
+        """
+        aggregate = self._get_resource(_aggregate.Aggregate, aggregate)
+        # We need to ensure we pass list of image IDs
+        if isinstance(images, str):
+            images = [images]
+        image_data = []
+        for img in images:
+            image_data.append({'id': img})
+        return aggregate.precache_images(self, image_data)
+
+    # ========== Images ==========
+
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
 
@@ -472,48 +519,53 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_keypair.Keypair, **attrs)
 
-    def delete_keypair(self, keypair, ignore_missing=True):
+    def delete_keypair(self, keypair, ignore_missing=True, user_id=None):
         """Delete a keypair
 
         :param keypair: The value can be either the ID of a keypair or a
-                        :class:`~openstack.compute.v2.keypair.Keypair`
-                        instance.
+            :class:`~openstack.compute.v2.keypair.Keypair` instance.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the keypair does not exist.
-                    When set to ``True``, no exception will be set when
-                    attempting to delete a nonexistent keypair.
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the keypair does not exist.  When set to ``True``, no exception
+            will be set when attempting to delete a nonexistent keypair.
+        :param str user_id: Optional user_id owning the keypair
 
         :returns: ``None``
         """
-        self._delete(_keypair.Keypair, keypair, ignore_missing=ignore_missing)
+        attrs = {'user_id': user_id} if user_id else {}
+        self._delete(_keypair.Keypair, keypair, ignore_missing=ignore_missing,
+                     **attrs)
 
-    def get_keypair(self, keypair):
+    def get_keypair(self, keypair, user_id=None):
         """Get a single keypair
 
         :param keypair: The value can be the ID of a keypair or a
-                        :class:`~openstack.compute.v2.keypair.Keypair`
-                        instance.
+            :class:`~openstack.compute.v2.keypair.Keypair` instance.
+        :param str user_id: Optional user_id owning the keypair
 
         :returns: One :class:`~openstack.compute.v2.keypair.Keypair`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
+            when no resource can be found.
         """
-        return self._get(_keypair.Keypair, keypair)
+        attrs = {'user_id': user_id} if user_id else {}
+        return self._get(_keypair.Keypair, keypair, **attrs)
 
-    def find_keypair(self, name_or_id, ignore_missing=True):
+    def find_keypair(self, name_or_id, ignore_missing=True, user_id=None):
         """Find a single keypair
 
         :param name_or_id: The name or ID of a keypair.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the resource does not exist.  When set to ``True``, None will be
+            returned when attempting to find a nonexistent resource.
+        :param str user_id: Optional user_id owning the keypair
+
         :returns: One :class:`~openstack.compute.v2.keypair.Keypair` or None
         """
+        attrs = {'user_id': user_id} if user_id else {}
         return self._find(_keypair.Keypair, name_or_id,
-                          ignore_missing=ignore_missing)
+                          ignore_missing=ignore_missing,
+                          **attrs)
 
     def keypairs(self, **query):
         """Return a generator of keypairs
