@@ -17,11 +17,11 @@ import os
 import sys
 
 import fixtures
+from io import StringIO
 import logging
 import munch
 from oslotest import base
 import pprint
-from six import StringIO
 import testtools.content
 
 _TRUE_VALUES = ('true', '1', 'yes')
@@ -41,9 +41,15 @@ class TestCase(base.BaseTestCase):
         # Do this before super setUp so that we intercept the default value
         # in oslotest. TODO(mordred) Make the default timeout configurable
         # in oslotest.
-        self.useFixture(
-            fixtures.EnvironmentVariable(
-                'OS_TEST_TIMEOUT', os.environ.get('OS_TEST_TIMEOUT', '5')))
+        test_timeout = int(os.environ.get('OS_TEST_TIMEOUT', '5'))
+        try:
+            test_timeout = int(test_timeout * self.TIMEOUT_SCALING_FACTOR)
+            self.useFixture(
+                fixtures.EnvironmentVariable(
+                    'OS_TEST_TIMEOUT', str(test_timeout)))
+        except ValueError:
+            # Let oslotest do its thing
+            pass
 
         super(TestCase, self).setUp()
 
@@ -65,8 +71,9 @@ class TestCase(base.BaseTestCase):
         logger.addHandler(handler)
 
         # Enable HTTP level tracing
+        # TODO(mordred) This is blowing out our memory we think
         logger = logging.getLogger('keystoneauth')
-        logger.setLevel(logging.DEBUG)
+        logger.setLevel(logging.INFO)
         logger.addHandler(handler)
         logger.propagate = False
 
