@@ -12,6 +12,7 @@
 from unittest import mock
 
 from openstack.compute.v2 import _proxy
+from openstack.compute.v2 import aggregate
 from openstack.compute.v2 import availability_zone as az
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor
@@ -41,6 +42,9 @@ class TestFlavor(TestComputeProxy):
     def test_flavor_delete(self):
         self.verify_delete(self.proxy.delete_flavor, flavor.Flavor, False)
 
+    def test_flavor_update(self):
+        self.verify_update(self.proxy.update_flavor, flavor.Flavor, False)
+
     def test_flavor_delete_ignore(self):
         self.verify_delete(self.proxy.delete_flavor, flavor.Flavor, True)
 
@@ -196,6 +200,112 @@ class TestFlavor(TestComputeProxy):
             expected_args=["prop"])
 
 
+class TestKeyPair(TestComputeProxy):
+    def test_keypair_create(self):
+        self.verify_create(self.proxy.create_keypair, keypair.Keypair)
+
+    def test_keypair_delete(self):
+        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, False)
+
+    def test_keypair_delete_ignore(self):
+        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, True)
+
+    def test_keypair_delete_user_id(self):
+        self.verify_delete(
+            self.proxy.delete_keypair, keypair.Keypair,
+            True,
+            method_kwargs={'user_id': 'fake_user'},
+            expected_kwargs={'user_id': 'fake_user'}
+        )
+
+    def test_keypair_find(self):
+        self.verify_find(self.proxy.find_keypair, keypair.Keypair)
+
+    def test_keypair_find_user_id(self):
+        self.verify_find(
+            self.proxy.find_keypair, keypair.Keypair,
+            method_kwargs={'user_id': 'fake_user'},
+            expected_kwargs={'user_id': 'fake_user'}
+        )
+
+    def test_keypair_get(self):
+        self.verify_get(self.proxy.get_keypair, keypair.Keypair)
+
+    def test_keypair_get_user_id(self):
+        self.verify_get(
+            self.proxy.get_keypair, keypair.Keypair,
+            method_kwargs={'user_id': 'fake_user'},
+            expected_kwargs={'user_id': 'fake_user'}
+        )
+
+    def test_keypairs(self):
+        self.verify_list_no_kwargs(self.proxy.keypairs, keypair.Keypair)
+
+    def test_keypairs_user_id(self):
+        self.verify_list(
+            self.proxy.keypairs, keypair.Keypair,
+            method_kwargs={'user_id': 'fake_user'},
+            expected_kwargs={'user_id': 'fake_user'}
+        )
+
+
+class TestAggregate(TestComputeProxy):
+    def test_aggregate_create(self):
+        self.verify_create(self.proxy.create_aggregate, aggregate.Aggregate)
+
+    def test_aggregate_delete(self):
+        self.verify_delete(
+            self.proxy.delete_aggregate, aggregate.Aggregate, False)
+
+    def test_aggregate_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_aggregate, aggregate.Aggregate, True)
+
+    def test_aggregate_find(self):
+        self.verify_find(self.proxy.find_aggregate, aggregate.Aggregate)
+
+    def test_aggregates(self):
+        self.verify_list_no_kwargs(self.proxy.aggregates, aggregate.Aggregate)
+
+    def test_aggregate_get(self):
+        self.verify_get(self.proxy.get_aggregate, aggregate.Aggregate)
+
+    def test_aggregate_update(self):
+        self.verify_update(self.proxy.update_aggregate, aggregate.Aggregate)
+
+    def test_aggregate_add_host(self):
+        self._verify("openstack.compute.v2.aggregate.Aggregate.add_host",
+                     self.proxy.add_host_to_aggregate,
+                     method_args=["value", "host"],
+                     expected_args=["host"])
+
+    def test_aggregate_remove_host(self):
+        self._verify("openstack.compute.v2.aggregate.Aggregate.remove_host",
+                     self.proxy.remove_host_from_aggregate,
+                     method_args=["value", "host"],
+                     expected_args=["host"])
+
+    def test_aggregate_set_metadata(self):
+        self._verify("openstack.compute.v2.aggregate.Aggregate.set_metadata",
+                     self.proxy.set_aggregate_metadata,
+                     method_args=["value", {'a': 'b'}],
+                     expected_args=[{'a': 'b'}])
+
+    def test_aggregate_precache_image(self):
+        self._verify(
+            "openstack.compute.v2.aggregate.Aggregate.precache_images",
+            self.proxy.aggregate_precache_images,
+            method_args=["value", '1'],
+            expected_args=[[{'id': '1'}]])
+
+    def test_aggregate_precache_images(self):
+        self._verify(
+            "openstack.compute.v2.aggregate.Aggregate.precache_images",
+            self.proxy.aggregate_precache_images,
+            method_args=["value", ['1', '2']],
+            expected_args=[[{'id': '1'}, {'id': '2'}]])
+
+
 class TestCompute(TestComputeProxy):
     def test_extension_find(self):
         self.verify_find(self.proxy.find_extension, extension.Extension)
@@ -225,24 +335,6 @@ class TestCompute(TestComputeProxy):
                          method_kwargs={"details": False, "query": 1},
                          expected_kwargs={"query": 1})
 
-    def test_keypair_create(self):
-        self.verify_create(self.proxy.create_keypair, keypair.Keypair)
-
-    def test_keypair_delete(self):
-        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, False)
-
-    def test_keypair_delete_ignore(self):
-        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, True)
-
-    def test_keypair_find(self):
-        self.verify_find(self.proxy.find_keypair, keypair.Keypair)
-
-    def test_keypair_get(self):
-        self.verify_get(self.proxy.get_keypair, keypair.Keypair)
-
-    def test_keypairs(self):
-        self.verify_list_no_kwargs(self.proxy.keypairs, keypair.Keypair)
-
     def test_limits_get(self):
         self.verify_get(self.proxy.get_limits, limits.Limits, value=[])
 
