@@ -32,16 +32,16 @@ class TestService(base.TestCase):
         self.assertEqual('service', sot.resource_key)
         self.assertEqual('services', sot.resources_key)
         self.assertEqual('/services', sot.base_path)
-        self.assertEqual('identity', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
-        self.assertEqual('PATCH', sot.update_method)
+        self.assertEqual('PATCH', sot.commit_method)
 
         self.assertDictEqual(
             {
+                'name': 'name',
                 'type': 'type',
                 'limit': 'limit',
                 'marker': 'marker',
